#!/bin/zsh

# Run this script to process the BAM files to correct misalignment of the W0 exons, extract details of IR1 alignment, and process the reads through the Matthews scripts for exon analysis.

# cd ~/[base directory in which to run scripts and place files]
# permissions with chmod u+x Shell_scripts/_Paper_analysis/vs_self_output/paper_Vs_self2_part2.sh
# run with ./Shell_scripts/_Paper_analysis/vs_self_output/paper_Vs_self2_part2.sh


# Script finds soft clips of 10-50 nt that map to W0 exon and remaps this soft clip onto the template.

Rscript ./Aaron_script/W0_correcting.r ./_Paper_analysis/vs_self_output/SAM_unq/AllvsSelf_unq.sam ./EBV_annotations/EBV_annotations_paper/EBNAexons_pHB9_oriPMlu.bed
# Moves output files to relevant folder [maybe no longer required].
mv updated_AllvsSelf_unq.sam _Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_corrected.sam
mv candidates_W0_id.sam _Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W0_id.sam
mv candidates_mixed_id.sam _Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_mixed_id.sam
mv candidates_W2_id.sam _Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W2_id.sam
mv candidates_unassigned_id.sam _Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_unassigned_id.sam
# Converts the W0-corrected SAM file to a BAM file, sort and index.
samtools view -b _Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_corrected.sam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_corrected.bam
samtools sort ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_corrected.bam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.bam
samtools index ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.bam
samtools view -h ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.bam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.sam
# remove SAM-> BAM conversion intermediates
rm ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_corrected.sam
rm ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_corrected.bam

# Make candidate reads available for viewing for checking on IGV - do not need to sort bam file as the Rscript re-sorts the SAM file before outputting them.
samtools view -b ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W0_id.sam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W0_id.bam
samtools index ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W0_id.bam

samtools view -b ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W2_id.sam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W2_id.bam
samtools index ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_W2_id.bam

samtools view -b ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_unassigned_id.sam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_unassigned_id.bam
samtools index ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_unassigned_id.bam

samtools view -b ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_mixed_id.sam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_mixed_id.bam
samtools index ./_Paper_analysis/vs_self_output/W0_SAM/W0_first/AllvsSelf_candidates_mixed_id.bam

#Now can run script to identify theoretical intact RNAs on W0-corrected SAM file - full_length_transcript_identification. 
# variables are: Window size for min number of reads to define canonical start/end; Min number of reads in that window; ±Distance from defined position to include in start; ±Distance after soft-clip restoration to include in start; 
Rscript ./Aaron_script/transcript_classifying.r ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.sam 2 5 20 20

# move output files - will overwrite files of same name
 mv 3p_intact_transcripts.sam _Paper_analysis/vs_self_output/W0_SAM/W0_3p_intact/AllvsSelf_3p_intact_transcripts.sam
 mv 5p_intact_transcripts.sam _Paper_analysis/vs_self_output/W0_SAM/W0_5p_intact/AllvsSelf_5p_intact_transcripts.sam
 mv unassigned.sam _Paper_analysis/vs_self_output/W0_SAM/W0_ends_not_assigned/AllvsSelf_unassigned.sam
 mv full_length_transcripts.sam _Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_full_length_transcripts.sam

# sort and index full length transcript subset Bam and sort SAM
samtools view -b ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_full_length_transcripts.sam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_full_length_transcripts.bam
samtools sort ././_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_full_length_transcripts.bam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.bam
samtools index ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.bam
samtools view -h ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.bam > ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.sam
# remove intermediates
rm ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_full_length_transcripts.sam
rm ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_full_length_transcripts.bam


# Run W repeat length analysis variation on the full length reads. Output to W0_SAM/W0_Full_length.
Rscript ./Aaron_script/IR1_repeat_counting.r ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.sam ./EBV_annotations/EBV_annotations_paper/EBNAexons_pHB9_oriPMlu.bed 20 3

mkdir ./_Paper_analysis/vs_self_output/IR1_analysis
mv exon_contents.txt _Paper_analysis/vs_self_output/IR1_analysis/AllvsSelf_fulllength_exon_contents.txt
mv IR1_repeat_counts.txt _Paper_analysis/vs_self_output/IR1_analysis/AllvsSelf_fulllength_IR1_repeat_counts.txt

# Run W repeat length analysis variation on all reads. Output to W0_SAM/W0_all_reads folder.

Rscript ./Aaron_script/IR1_repeat_counting.r ./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.sam ./EBV_annotations/EBV_annotations_paper/EBNAexons_pHB9_oriPMlu.bed 20 3

mv exon_contents.txt _Paper_analysis/vs_self_output/IR1_analysis/AllvsSelf_AllReads_exon_contents.txt
mv IR1_repeat_counts.txt _Paper_analysis/vs_self_output/IR1_analysis/AllvsSelf__AllReads_IR1_repeat_counts.txt


   # Use nanopolish index to link fastq reads to fast5 files for nanopolish. Takes a relatively long time, so should only need to do this once. Can use the FAST5 folder of separated EBV reads generated above.
   #[readdb] num reads: 3689846, num reads with path to fast5: 3693
   #fast5 files could not be located for 3686153 reads
    
nanopolish index -d EBV_oriPMlu_Fast5 ./Guppy_6_output/Guppy6_Allpassreads.fastq.gz

# Run nanopolish to output the pA status of all of the reads. Needs a bam file input. 
nanopolish polya --threads=8 --reads=Guppy_6_output/Guppy6_Allpassreads.fastq.gz --bam=./_Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.bam --genome=EBV_annotations/EBV_annotations_paper/MR11_oriPMlu.fa > _Paper_analysis/vs_self_output/Matthews_script/Nanopolish/AllvsSelf_W0_polya_results.tsv

# Run the Matthews perl script 1 - classify_transcripts_and_polya_segmented_V2.pl. Outputs prefixed 1_
# - guidance:classify_transcripts_and_polya_segmented_V2.pl prefix polya/TSS_window splice_window polya_min nanopolish_polya.txt input.sam genome.fasta splice_tolerance(1) 100
# changed numbers to use: start/pA window +/-20; splice window 2; minpA 8 as there are positionally realistic reads with short pA.
# using an altered version of the MR11 fasta (MR11_noATG) to remove ATGs that are not used or make very small peptides to help EBNA protein sequence outputs.

perl Matthews_script/classify_transcripts_and_polya_segmented_V2.pl _Paper_analysis/vs_self_output/Matthews_script/Script_out/1_AllvsSelf_W0_AllReads 20 2 8 _Paper_analysis/vs_self_output/Matthews_script/Nanopolish/AllvsSelf_W0_polya_results.tsv _Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.sam EBV_annotations/EBV_annotations_paper/MR11_oriPMlu_noATG.fa 1 80

# Run the Matthews script 2 for All Reads. file name does not tolerate folder prefix. All output files prefixed with 2_
# - guidance: And now second script: name_transcripts_and_track_ssc_V2.pl prefix ssc_nt features_table.txt genome.fasta start_sad_stop_pattern_count.txt
perl Matthews_script/name_transcripts_and_track_ssc_V2.pl 2_AllvsSelf_W0_AllReads 10 EBV_annotations/EBV_annotations_paper/Features_table_pHB9_oriPMlu.txt EBV_annotations/EBV_annotations_paper/MR11_oriPMlu_noATG.fa _Paper_analysis/vs_self_output/Matthews_script/Script_out/1_AllvsSelf_W0_AllReads.start_sad_stop_pattern_count.txt
# move output files to appropriate locations
mv 2_AllvsSelf_W0_AllReads_ssc.proteins_not_known.fasta _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_AllReads_ssc.proteins_not_known.fasta
mv 2_AllvsSelf_W0_AllReads_ssc.proteins_known.fasta _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_AllReads_ssc.proteins_known.fasta
cp -r 2_AllvsSelf_W0_AllReads_ssc  _Paper_analysis/vs_self_output/Matthews_script/Script_out/
rm -rf 2_AllvsSelf_W0_AllReads_ssc
mv 2_AllvsSelf_W0_AllReads_ssc.combined_counts_and_feature_names.txt  _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_AllReads_ssc.combined_counts_and_feature_names.txt
mv 2_AllvsSelf_W0_AllReads_ssc.count_of_translated_features.txt  _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_AllReads_ssc.count_of_translated_features.txt
mv 2_AllvsSelf_W0_AllReads_ssc.most_abundant_trans_per_orf.gff _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_AllReads_ssc.most_abundant_trans_per_orf.gff
mv 2_AllvsSelf_W0_AllReads_ssc.splice_acceptor_donor_usage.txt _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_AllReads_ssc.splice_acceptor_donor_usage.txt
mv 2_AllvsSelf_W0_AllReads_ssc.TSS_start_sad_polyA_count_list.txt _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_AllReads_ssc.TSS_start_sad_polyA_count_list.txt

# final part of Matthews pipeline: not used in this study's analyses 
perl Matthews_script/all_transcripts_grouped_by_polya.pl _Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.sam 1 100 20 25 40 EBV_annotations/EBV_annotations_paper/MR11_oriPMlu_noATG.fa

mkdir _Paper_analysis/vs_self_output/Matthews_script/Script_out/pA_SAM_allreads
mv -f polya_group_* _Paper_analysis/vs_self_output/Matthews_script/Script_out/pA_SAM_allreads


# Now re-run the Matthews scripts for the full length reads only.
# - guidance:classify_transcripts_and_polya_segmented_V2.pl prefix polya/TSS_window splice_window polya_min nanopolish_polya.txt input.sam genome.fasta splice_tolerance(1) 100

perl Matthews_script/classify_transcripts_and_polya_segmented_V2.pl _Paper_analysis/vs_self_output/Matthews_script/Script_out/1_AllvsSelf_W0_fullLength 20 2 8 _Paper_analysis/vs_self_output/Matthews_script/Nanopolish/AllvsSelf_W0_polya_results.tsv ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.sam EBV_annotations/EBV_annotations_paper/MR11_oriPMlu_noATG.fa 1 80

# Run the Matthews script 2. 
# - guidance: And now second script: name_transcripts_and_track_ssc_V2.pl prefix ssc_nt features_table.txt genome.fasta start_sad_stop_pattern_count.txt
perl Matthews_script/name_transcripts_and_track_ssc_V2.pl 2_AllvsSelf_W0_fullLength 10 EBV_annotations/EBV_annotations_paper/Features_table_pHB9_oriPMlu.txt EBV_annotations/EBV_annotations_paper/MR11_oriPMlu_noATG.fa _Paper_analysis/vs_self_output/Matthews_script/Script_out/1_AllvsSelf_W0_fullLength.start_sad_stop_pattern_count.txt


mv 2_AllvsSelf_W0_fullLength_ssc.proteins_not_known.fasta _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_fullLength_ssc.proteins_not_known.fasta
mv 2_AllvsSelf_W0_fullLength_ssc.proteins_known.fasta  _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_fullLength_ssc.proteins_known.fasta
cp -r 2_AllvsSelf_W0_fullLength_ssc _Paper_analysis/vs_self_output/Matthews_script/Script_out/
rm -rf 2_AllvsSelf_W0_fullLength_ssc 
mv 2_AllvsSelf_W0_fullLength_ssc.combined_counts_and_feature_names.txt  _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_fullLength_ssc.combined_counts_and_feature_names.txt
mv 2_AllvsSelf_W0_fullLength_ssc.count_of_translated_features.txt _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_fullLength_ssc.count_of_translated_features.txt
mv 2_AllvsSelf_W0_fullLength_ssc.most_abundant_trans_per_orf.gff  _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_fullLength_ssc.most_abundant_trans_per_orf.gff
mv 2_AllvsSelf_W0_fullLength_ssc.splice_acceptor_donor_usage.txt _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_fullLength_ssc.splice_acceptor_donor_usage.txt
mv 2_AllvsSelf_W0_fullLength_ssc.TSS_start_sad_polyA_count_list.txt _Paper_analysis/vs_self_output/Matthews_script/Script_out/2_AllvsSelf_W0_fullLength_ssc.TSS_start_sad_polyA_count_list.txt

perl Matthews_script/all_transcripts_grouped_by_polya.pl ./_Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.sam 1 100 20 25 40 EBV_annotations/EBV_annotations_paper/MR11_oriPMlu_noATG.fa

mkdir _Paper_analysis/vs_self_output/Matthews_script/Script_out/pA_SAM_full_length
mv -f polya_group_* _Paper_analysis/vs_self_output/Matthews_script/Script_out/pA_SAM_full_length

# copy files to be ready to repeat the analysis separately for each sample.

mkdir _Paper_analysis/vs_self_output/Single_samples
cp _Paper_analysis/vs_self_output/W0_SAM/W0_all_reads/AllvsSelf_W0_sort.sam  _Paper_analysis/vs_self_output/Single_samples/AllvsSelf_W0_sort.sam
cp _Paper_analysis/vs_self_output/W0_SAM/W0_full_length/AllvsSelf_W0_fullLength_sort.sam  _Paper_analysis/vs_self_output/Single_samples/AllvsSelf_W0_fullLength_sort.sam

